#include <stdint.h>
#include <iostream>

#include "chr_chronos.h"

#define SENSOR_ID(key) (*((uint32_t*) ((unsigned char*)(key))))
#define TIMESTAMP(key) (*((uint64_t*) (((unsigned char*)(key)) + sizeof(uint32_t))))
#define VALUE(value)   (*((double*)   ((unsigned char*)(value))))

using namespace std;

int main(int argc, const char * argv[])
{
  /* init a database */
  chr_chronos * chronos = new chr_chronos("/tmp/chronos_dummy_flash_device",
      "/tmp/chronos_dummy_persistence_folder/");

  /* create a table */
  chronos->create_table("samples", sizeof(uint32_t) + sizeof(uint64_t), sizeof(double));

  /* open a table */
  chr_table_ctx * samples_ctx = chronos->open_table("samples");

  unsigned char * key = (unsigned char *) malloc(sizeof(uint32_t) + sizeof(uint64_t));
  unsigned char * value = (unsigned char *) malloc(sizeof(double));
  /* assign values to key, in big-endian form */
  SENSOR_ID(key) = htobe32(10);
  TIMESTAMP(key) = htobe64(1224598516123);
  VALUE(value) = 3.14;

  /* open a cursor */
  chr_cursor * cur = samples_ctx->open_cursor(key, CHR_KEY);

  /* insert a key-value pair */
  cur->insert(key, value);

  VALUE(value) = 1.62;

  /* update a value */
  cur->update(key, value);

  /* move a cursor and read key-value pairs */
  cur->move(NULL, CHR_FIRST, CHR_BEFORE);
  while (cur->read_next(key, value) != CHR_EOF)
  {
    uint32_t sample_id = be32toh(SENSOR_ID(key));
    uint64_t timestamp = be64toh(TIMESTAMP(key));
    double sample = VALUE(value);
    cout << sample_id << "," << timestamp << "," << sample << endl;
  }

  /* delete a key-value pair */
  cur->del(key);

  /* close a cursor */
  samples_ctx->close_cursor(cur);

  /* close a table */
  chronos->close_table(samples_ctx);

  /* drop a table */
  chronos->drop_table("samples");

  /* close a database */
  delete chronos;
}

