#ifndef CHR_LEAF_H
#define CHR_LEAF_H

#include "chr_common.h"

#include "chr_flash.h"
#include "chr_node.h"
#include "chr_inode.h"

class chr_leaf : public chr_node
{
public:
  chr_leaf(size_t key_size, size_t value_size,
      chr_inode * parent, chr_table * table, chr_flash * flash,
      bool kmininf, const unsigned char * kmin, bool kmaxinf,
      const unsigned char * kmax, chr_lsn lsn,
      const unsigned char * buf);
  ~chr_leaf();
  
  unsigned char * value(unsigned int x);
  int insert(const unsigned char * key, const unsigned char * value);
  int update(const unsigned char * key, const unsigned char * value);
  int del(const unsigned char * key);
  int readnext(const unsigned char * prevkey, unsigned char * k,
      unsigned char * val, int pos_hint);
  int readprev(const unsigned char * prevkey, unsigned char * k,
      unsigned char * val, int pos_hint);

  int read_exact(const unsigned char * key, unsigned char * value);
  int read_exactornext(unsigned char * key, unsigned char * value);
  int read_exactorprev(unsigned char * key, unsigned char * value);
  int read_first(unsigned char * key, unsigned char * value);
  int read_last(unsigned char * key, unsigned char * value);
  void drop();

  bool is_leaf();
  chr_leaf * leaf();
  chr_inode * inode();

  void dbug_check(bool kmininf, const unsigned char * kmin,
      bool kmaxinf, const unsigned char * kmax);

  chr_lsn lsn;

  void get_state(chr_lsn * lsn, bool * kmininf, unsigned char ** kmin,
      bool * kmaxinf, unsigned char ** kmax, size_t * key_size,
      size_t * value_size, unsigned int * nb_keys, unsigned char ** keys,
      unsigned char ** values);

private:
  chr_leaf(const chr_leaf &);
  chr_leaf & operator=(const chr_leaf &);

  void to_sector(unsigned char * buf);
  int split();

  const size_t value_size;
  unsigned char * max_key_epsilon;
  /* max position in keys array of last 'epsilon' inserts
   * used to determine the "median" key during split */
  unsigned char * values;
  unsigned char * buf;

  bool dirty;
};

#endif /* CHR_LEAF_H */

