#ifndef CHR_LIST_H
#define CHR_LIST_H

#include "chr_common.h"

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <assert.h>

#include <set>

template <typename E, E nullvalue>
class chr_list {
public:
  chr_list();
  ~chr_list();
  void insert(const E);
  void remove(const E);
  E pop();
  unsigned long long size();
private:
  std::set<E> list;
  typename std::set<E>::iterator cur;
  chr_list(const chr_list<E, nullvalue>&);
  chr_list<E, nullvalue>& operator=(const chr_list<E, nullvalue>&);
};


template <typename E, E nullvalue>
chr_list<E, nullvalue>::chr_list()
{
  cur = list.begin();
}

template <typename E, E nullvalue>
chr_list<E, nullvalue>::~chr_list()
{

}

template <typename E, E nullvalue>
void chr_list<E, nullvalue>::insert(const E val)
{
  assert(val != nullvalue);
  /* ensure no double trim */
  assert(list.find(val) == list.end());
  list.insert(val);
}

template <typename E, E nullvalue>
void chr_list<E, nullvalue>::remove(const E val)
{
  list.erase(val);
}

template <typename E, E nullvalue>
unsigned long long chr_list<E, nullvalue>::size()
{
  return(list.size());
}

template <typename E, E nullvalue>
E chr_list<E, nullvalue>::pop()
{
  if(cur == list.end())
  {
    cur = list.begin();
    if(cur == list.end())
    {
      return(nullvalue);
    }
  }

  E ret = *cur;
  list.erase(cur++);
  return(ret);
}

#ifdef NO_IMPLICIT_TEMPLATES
  #include "chr_lsn.h"
  template class chr_list<chr_lsn>;
  template class std::_Rb_tree
    <
      unsigned long long,
      unsigned long long,
      std::_Identity<unsigned long long>,
      std::less<unsigned long long>,
      std::allocator<unsigned long long>
    >;
#endif

#endif /* CHR_LIST_H */

